﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Threading;
using System.Windows.Media;
using DPackLib;

namespace Generators
{
    public partial class GeneratorPanel
    {
        public string PanelName { get; set; }
        private const int incrStep = 25;
        private int period { get; set; }
        private bool UseWcf { get; set; }
        private bool isStarted { get; set; }
        BinSerializer serializer { get; set; }
        private DispatcherTimer dispatcherTimer;
        private DispatcherTimer indicatorTimer;
        private DispatcherTimer alertTimer;

        public GeneratorPanel()
        {
            InitializeComponent();
        }

        public void SetChannel(bool useWcf)
        {
            UseWcf = useWcf;
        }

        private void GeneratorPanel_OnLoaded(object sender, RoutedEventArgs e)
        {
            DescriptionTxt.Text = PanelName;
            serializer= new BinSerializer();
        }

        public void CreateTimers(int sendPeriod)
        {
            period = sendPeriod;
            PeriodTxt.Text = period.ToString();
            dispatcherTimer = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds(period) };
            dispatcherTimer.Tick += Timer_Tick;
            dispatcherTimer.Start();
            isStarted = true;

            indicatorTimer = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds(50) };
            indicatorTimer.Tick += Indicator_Tick;

            alertTimer = new DispatcherTimer { Interval = TimeSpan.FromSeconds(1) };
            alertTimer.Tick += AlertTimer_Tick;
            alertTimer.Start();
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            Indicator.Background = Brushes.OrangeRed;
            indicatorTimer.Start();
            if (UseWcf) { SendDataWcf(true); } else { SendDataLocal(true); }
        }

        private void AlertTimer_Tick(object sender, EventArgs e)
        {
            if (!Directory.Exists(CommonConstants.RootDir)) return;
            var files = Directory.GetFiles(CommonConstants.RootDir);
            foreach (var file in files)
            {
                var flNm = Path.GetFileName(file);
                if (flNm == PanelName + "_rt.dat")
                {
                    ReadAlertData(file);
                }
            }
        }

        private void SendDataLocal(bool isMock)
        {
            var dp = PrepareDataPack(isMock);            
            var serialiser = new BinSerializer();
            var path = Path.Combine(CommonConstants.RootDir, DescriptionTxt.Text + ".dat");
            serialiser.SerializeData(dp, path);
        }

        private void SendDataWcf(bool isMock)
        {

        }

        private DataPack PrepareDataPack(bool isMock)
        {
            if (isMock)
            {
                DataPack dpm = new MockDataMaker().CreateMockDataPack(PanelName);
                MessageTxt.Text = dpm.Message;
                DataTxt.Text = dpm.Data.ToString("0.0");
                return dpm;
            }

            var dp = new DataPack();
            dp.PackDescription = PanelName;
            dp.Data =  Convert.ToDouble(DataTxt.Text);
            dp.Message =  MessageTxt.Text;
            dp.DateTimeTicks = DateTime.Now.Ticks;
            dp.IsReturned = false;
          
            MessageTxt.Text = dp.Message;
            DataTxt.Text = dp.Data.ToString("0.0");
            return dp;
        }

        private void Indicator_Tick(object sender, EventArgs e)
        {
            Indicator.Background = Brushes.DarkSlateGray;
            indicatorTimer.Stop();
        }

        private void ReadAlertData(string path)
        {
            if (!File.Exists(path)) return;

            DataPack dataPack = serializer.DeSerializeData(path);
            if (dataPack == null) return;

            var delay = new TimeSpan(DateTime.Now.Ticks - dataPack.DateTimeTicks).Seconds;

            if (delay > 58)
            {
                try { File.Delete(path); } catch { }        
                AlertTxt.Text = "";
                AlertTxt.Visibility = Visibility.Collapsed;
                Height = 50;
                return;
            }

            AlertTxt.Text = dataPack.Message + "-" 
                + new TimeSpan(DateTime.Now.Ticks - dataPack.DateTimeTicks).Seconds + " sec ago";
            AlertTxt.Visibility=Visibility.Visible;
            Height = 80;
        }

        private void PlusPeriod_Click(object sender, RoutedEventArgs e)
        {
            period += incrStep;
            PeriodTxt.Text = period.ToString();
            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(period);
        }

        private void MinusPeriod_Click(object sender, RoutedEventArgs e)
        {
            if (period > incrStep) { period -= incrStep; }
            PeriodTxt.Text = period.ToString();
            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(period);
        }

        private void BtSend_Click(object sender, RoutedEventArgs e)
        {

            if (RandomCheck.IsChecked != true)
            {
               if (UseWcf) { SendDataWcf(false); } else { SendDataLocal(false); }
            }
            else
            {
                RandomCheck.IsChecked = false;
                dispatcherTimer.Stop();  
            }       
        }

        private void RandomCheck_OnChecked(object sender, RoutedEventArgs e)
        {
            dispatcherTimer?.Start();
        }

        private void RandomCheck_Unchecked(object sender, RoutedEventArgs e)
        {
            dispatcherTimer?.Stop();
        }
    }

}
