﻿using System;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using DPackLib;

namespace Generators
{
    public partial class MainWindow 
    {
        Random rnd = new Random();

        public MainWindow()
        {
            InitializeComponent();
        }

        private void MainWindow_OnLoaded(object sender, RoutedEventArgs e)
        {
            foreach (var knownType in CommonConstants.DetectorsTypes)
            {
                var itm = new ListBoxItem();
                itm.Foreground= Brushes.Moccasin;
                itm.Background = Brushes.Black;
                itm.Content = knownType;
                GeneratorsList.Items.Add(itm);
                AddGenerator(knownType);
            }

            if (GeneratorsList.Items.Count > 0)
            { GeneratorsList.SelectedIndex = 0; }
        }

        private void AddGenerator_OnClick(object sender, RoutedEventArgs e)
        {
            var itm=(ListBoxItem)GeneratorsList.SelectedItem;

            string genType =(string) itm.Content;
            if (string.IsNullOrEmpty(genType)) { genType = "Speed"; }

            AddGenerator(genType);
        }

        private void AddGenerator(string genType)
        {
            int nr = 1;

            foreach (var children in GeneratopStack.Children)
            {
                var gen = (GeneratorPanel)children;
                if (gen != null && gen.PanelName.Contains(genType)) { nr++; }
            }

            var genPanel = new GeneratorPanel
            {
                PanelName = nr + "-" + genType,
                Margin = new Thickness(0, 5, 0, 5)
            };

            int rn = rnd.Next(1, 10)*25;
            genPanel.CreateTimers(rn);
            GeneratopStack.Children.Add(genPanel);
        }

        private void CleanData()
        {
            if (!Directory.Exists(CommonConstants.RootDir)) return;

            var datFiles = Directory.GetFiles(CommonConstants.RootDir).Where(x => x.EndsWith(".dat"));
            foreach (var datFile in datFiles)
            {
                File.Delete(datFile);
            }
        }

        private void MainWindow_OnClosing(object sender, CancelEventArgs e)
        {
            CleanData();
        }
    }
}
