﻿using System;
using System.IO;
using System.Windows.Threading;
using DPackLib;

namespace Monitor.Controls
{

    public partial class FuelPanel 
    {

        public string Description { get; set; }
        private string dataPath { get; set; }
        private DispatcherTimer resTimer;
        BinSerializer serializer { get; set; }
        private double fuel = 20;
        private long lastTick = 0; 

        public FuelPanel()
        {
            InitializeComponent();
        }

        public void StartReception(int period = 100)
        {
            DescriptionTxt.Text = Description;
            dataPath = Path.Combine(CommonConstants.RootDir, Description + ".dat");
            serializer = new BinSerializer();
            CreateTimer(period);
        }

        private void CreateTimer(int period)
        {
            resTimer = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds(period) };
            resTimer.Tick += GenTimer_Tick;
            resTimer.Start();
        }

        private void GenTimer_Tick(object sender, EventArgs e)
        {
            ReadData();
        }

        private void ReadData()
        {
            DataPack dataPack = serializer.DeSerializeData(dataPath);
            if (dataPack == null ) return;

            var delay = new TimeSpan(DateTime.Now.Ticks - dataPack.DateTimeTicks);
            DelayTxt.Text = delay.Minutes >= 1 ? "Line is inactive" : $"Delay: {delay.TotalMilliseconds:0} ms";

            if (dataPack.DateTimeTicks == lastTick) return;
            lastTick = dataPack.DateTimeTicks;

            InfoTxt.Text = dataPack.Message;
            if(fuel>0){ fuel -= dataPack.Data / 50;}

            FuelIndicator.Value = fuel;
            FuelTxt.Text = fuel.ToString("0.0") + " Liters";
        }

    }
}
