﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using System.Windows.Media;
using DPackLib;
using Path = System.IO.Path;


namespace Monitor.Controls
{

    public partial class PathPanel 
    {
        public string Description { get; set; }
        private string dataPath { get; set; }
        private DispatcherTimer resTimer;
        BinSerializer serializer { get; set; }
        private double remain = 305;
        private double location = 0;
        private double xShift = -3, yShift = 26;
        List<Point> PathPointsList = new List<Point>();
        private long lastTick = 0;

        public PathPanel()
        {
            InitializeComponent();
        }

        public void StartReception(int period = 100)
        {
            DescriptionTxt.Text = Description;
            dataPath = Path.Combine(CommonConstants.RootDir, Description + ".dat");
            serializer = new BinSerializer();
            CreateTimer(period);
            PathPointsList = GetPathPoints(track);
        }

        public void CreateTimer(int period )
        {
            resTimer = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds(period) };
            resTimer.Tick += GenTimer_Tick;
            resTimer.Start();
        }

        private void GenTimer_Tick(object sender, EventArgs e)
        {
            ReadData();
        }

        private List<Point> GetPathPoints(System.Windows.Shapes.Path path)
        {
            Point p;
            Point tg;
            var pointsList = new List<Point>();

            var pathGeometry = path.Data.GetFlattenedPathGeometry(0.01, ToleranceType.Absolute);
    
            for (var i = 0; i < 1000; i++)
            {
                pathGeometry.GetPointAtFractionLength(i / 1000f, out p, out tg);
                pointsList.Add(p);
            }

            return pointsList;
        }

        private void ReadData()
        {
            DataPack dataPack = serializer.DeSerializeData(dataPath);
            if (dataPack == null) return;

            var delay = new TimeSpan(DateTime.Now.Ticks - dataPack.DateTimeTicks);
            DelayTxt.Text = delay.Minutes >= 1 ? "Line is inactive" : $"Delay: {delay.TotalMilliseconds:0} ms";

            if (dataPack.DateTimeTicks == lastTick) return;
            lastTick = dataPack.DateTimeTicks;

            InfoTxt.Text = dataPack.Message;

            if (remain > 0){ remain -= dataPack.Data / 2; }
            PathTxt.Text = remain.ToString("0") + " km to final";

            int idx = (int)(999 - (remain/305)*1000);
            var point = PathPointsList[idx];           
            Canvas.SetLeft(CarIndicator, point.X + xShift);
            Canvas.SetTop(CarIndicator, point.Y+ yShift);         
            CarIndicator.PointToScreen(point);
        }
    }
}
