﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Windows;
using System.Windows.Threading;
using DPackLib;
using Microsoft.Practices.Unity;
using Prism.Events;
using Telerik.Charting;

namespace Monitor.Controls
{
    public partial class PilotPanel 
    {
        [Dependency] public IEventAggregator EventAggregator { get; set; }
        public string Description { get; set; }

        #region private Var

        private string dataPath { get; set; }
        private DispatcherTimer resTimer;
        List<string> messageList = new List<string>();
        BinSerializer serializer { get; set; }
        private string lastMessage { get; set; }
        private long lastTick = 0;
        private bool maxValReached = false;
        private double tempVal = 120.0;

        #endregion
        public PilotPanel()
        {
            InitializeComponent();
        }

        public void StartReception(int period=100)
        {
            DescriptionTxt.Text = Description;
            dataPath = Path.Combine(CommonConstants.RootDir, Description + ".dat");
            serializer = new BinSerializer();
            CreateTimer(period);
        }

        private void CreateTimer(int period )
        {
            resTimer = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds(period) };
            resTimer.Tick += GenTimer_Tick;
            resTimer.Start();
        }

        private void GenTimer_Tick(object sender, EventArgs e)
        {
            ReadData();
        }

        private void ReadData()
        {
            DataPack dataPack = serializer.DeSerializeData(dataPath);
            if (dataPack == null) return;

            var delay = new TimeSpan(DateTime.Now.Ticks - dataPack.DateTimeTicks);
            DelayTxt.Text = delay.Minutes >= 1 ? "Line is inactive" : $"Delay: {delay.TotalMilliseconds:0} ms";

            if (dataPack.DateTimeTicks == lastTick || lastMessage == dataPack.Message) return;
            lastTick = dataPack.DateTimeTicks;

            if (maxValReached)
            {
                tempVal -= dataPack.Data * 50;
                if (tempVal <= 100)
                {
                    tempVal = 100;
                    maxValReached = false;
                }
            }
            else
            {
                tempVal += dataPack.Data * 50;
                if (tempVal > 250)
                {
                    tempVal = 300;
                    maxValReached = true;
                }
            }

            LnSeries.DataPoints.Add(new CategoricalDataPoint() { Value = tempVal });
            if (LnSeries.DataPoints.Count > 10){ LnSeries.DataPoints.RemoveAt(0);}

            lastMessage = dataPack.Message;
            messageList.Add(dataPack.Message);
            if (messageList.Count > 5)
            {
                messageList = Enumerable.Reverse(messageList).Take(5).Reverse().ToList();
            }
            PilotText.Text = string.Join("\r\n", messageList.ToArray());
            PilotText.ScrollToEnd();
        }

        private void BtnSendMessage_Click(object sender, RoutedEventArgs e)
        {
            SendDataLocal();
        }

        private void SendDataLocal()
        {
            var dp = new DataPack();
            dp.PackDescription = Description;
            dp.Data = 0.0;
            dp.Message = MessageText.Text;
            dp.DateTimeTicks = DateTime.Now.Ticks;
            dp.IsReturned = true;
            var serialiser = new BinSerializer();
            var path = Path.Combine(CommonConstants.RootDir, DescriptionTxt.Text + "_rt.dat");
            serialiser.SerializeData(dp, path);
        }

    }
}
