﻿using System;
using System.IO;
using System.Windows.Threading;
using DPackLib;

namespace Monitor.Controls
{
    public partial class SpeedPanel 
    {
        public string Description { get; set; }

        #region Private Var

        private BinSerializer serializer { get; set; }
        private string dataPath { get; set; }
        private DispatcherTimer resTimer;
        private double tempSpeed =0;
        private bool maxSpeedReached = false;
        private long lastTick = 0;

        #endregion

        public SpeedPanel()
        {
            InitializeComponent();
        }

        public void StartReception(int period = 100)
        {
            DescriptionTxt.Text = Description;
            dataPath = Path.Combine(CommonConstants.RootDir, Description + ".dat");
            serializer = new BinSerializer();
            CreateTimer(period);
        }

        private void CreateTimer(int period)
        {
            resTimer = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds(period) };
            resTimer.Tick += GenTimer_Tick;
            resTimer.Start();
        }

        private void GenTimer_Tick(object sender, EventArgs e)
        {
            ReadData();
        }

        private void ReadData()
        {
            DataPack dataPack = serializer.DeSerializeData(dataPath);
            if (dataPack == null) return;

            var delay = new TimeSpan(DateTime.Now.Ticks - dataPack.DateTimeTicks);
            DelayTxt.Text = delay.Minutes >= 1 ? "Line is inactive" : $"Delay: {delay.TotalMilliseconds:0} ms";

            if (dataPack.DateTimeTicks == lastTick) return;
            lastTick = dataPack.DateTimeTicks;

            InfoTxt.Text = dataPack.Message;

            if (maxSpeedReached)
            {
                tempSpeed -= dataPack.Data*20;
                if (tempSpeed <= 0)
                {
                    tempSpeed = 0;
                    maxSpeedReached = false;
                }
            }
            else
            {
                tempSpeed += dataPack.Data * 20;
                if (tempSpeed > 300) {maxSpeedReached = true;}
            }

            SpeedNeedle.Value = tempSpeed;
            SpeedTxt.Text = tempSpeed.ToString("0")+ " km/h";
        }
    }
}
