﻿using System;
using System.IO;
using System.Windows.Threading;
using DPackLib;

namespace Monitor.Controls
{
    public partial class TemperaturePanel 
    {
        public string Description { get; set; }
        private string dataPath { get; set; }
        private DispatcherTimer resTimer;
        BinSerializer serializer { get; set; }
        private double tempTmp = 0;
        private bool maxTmpReached = false;
        private long lastTick = 0;

        public TemperaturePanel()
        {
            InitializeComponent();
        }

        public void StartReception(int period = 100)
        {
            DescriptionTxt.Text = Description;
            dataPath = Path.Combine(CommonConstants.RootDir, Description + ".dat");
            serializer = new BinSerializer();
            CreateTimer(period);
        }

        private void CreateTimer(int period)
        {
            resTimer = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds(period) };
            resTimer.Tick += GenTimer_Tick;
            resTimer.Start();
        }

        private void GenTimer_Tick(object sender, EventArgs e)
        {
            ReadData();
        }

        private void ReadData()
        {
            DataPack dataPack = serializer.DeSerializeData(dataPath);
            if (dataPack == null) return;

            var delay = new TimeSpan(DateTime.Now.Ticks - dataPack.DateTimeTicks);
            DelayTxt.Text = delay.Minutes >= 1 ? "Line is inactive" : $"Delay: {delay.TotalMilliseconds:0} ms";

            if (dataPack.DateTimeTicks == lastTick) return;
            lastTick = dataPack.DateTimeTicks;

            InfoTxt.Text = dataPack.Message;

            if (maxTmpReached)
            {
                tempTmp -= dataPack.Data * 5;
                if (tempTmp <= 0)
                {
                    tempTmp = 0;
                    maxTmpReached = false;
                }
            }
            else
            {
                tempTmp += dataPack.Data * 5;
                if (tempTmp > 120) { maxTmpReached = true; }
            }


            TemperatureBar.Value = tempTmp;
            TemperatureTxt.Text = tempTmp.ToString("0") + " °C";
        }
    }
}
