﻿using System.Windows;
using Monitor.Interfaces;
using Microsoft.Practices.Unity;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Regions;
using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading;
using System.Windows.Threading;
using DPackLib;

namespace Monitor.ViewModels
{
    public class ShellViewModel : BindableBase, IShellViewModel, IDisposable
    {
        [Dependency]public IRegionManager RegionManager { get; set; }
        [Dependency]public IUnityContainer Container { get; set; }
        [Dependency]public IEventAggregator EventAggregator { get; set; }

        public DelegateCommand<string> NavigateCommand { get; set; }
        private Process genProcess { get; set; }
        private DispatcherTimer startTimer;
        private int tm = 0;

        [InjectionMethod]
        public void InitializeShellView()
        {
            NavigateCommand = new DelegateCommand<string>(Navigate);         
            CreateStartTimer();
        }

        private void CreateStartTimer()
        {
            startTimer = new DispatcherTimer { Interval = TimeSpan.FromSeconds(1) };
            startTimer.Tick += GenTimer_Tick;
            startTimer.Start();
        }

        private void GenTimer_Tick(object sender, EventArgs e)
        {
            tm++;
            if (tm==1) { Navigate("StartText");}
            if(tm >=64)
            {
                startTimer.Stop();
                Navigate("StartGenerators");
            }          
        }

        private void Navigate(string uri)
        {
            foreach (var view in RegionManager.Regions["MainRegion"].Views)
            { RegionManager.Regions["MainRegion"].Remove(view); }

            switch (uri)
            {
                case "StartText":
                    RegionManager.AddToRegion("MainRegion", Container.Resolve<IAnim3DtextViewModel>());
                    break;

                case "StartGenerators":
                    RegionManager.AddToRegion("MainRegion", Container.Resolve<IMonitorsSetViewModel>());
                    if (File.Exists(CommonConstants.GeneratorsPath))
                    {
                        try { genProcess = StartProc(CommonConstants.GeneratorsPath);} 
                        catch { MessageBox.Show(CommonConstants.GeneratorsPath + "\r\nNot Started!");}
                    }
                    else
                    {
                        try
                        {
                            Directory.CreateDirectory(Path.GetDirectoryName(CommonConstants.GeneratorsPath));
                            File.Copy(Path.GetFileName(CommonConstants.GeneratorsPath), CommonConstants.GeneratorsPath);
                            File.Copy(Path.GetFileName(CommonConstants.LibPath), CommonConstants.LibPath);
                            genProcess = StartProc(CommonConstants.GeneratorsPath);
                        }
                        catch { MessageBox.Show(Path.GetDirectoryName(CommonConstants.GeneratorsPath) + "\r\nNot Copied!"); }
                    }
                    startTimer.Stop();
                    break;

                case "Shutdown":
                    if (genProcess != null) { StopProc(genProcess); }                    
                    Application.Current.Shutdown();
                    break;
            }
        }

        public Process StartProc(string procname)
        {
            var proc = new Process {StartInfo = {FileName = procname}};
            proc.Start();
            return proc;
        }

        public void StopProc(Process proc)
        {
            proc.CloseMainWindow();

            Thread.Sleep(1000);

            if (Directory.Exists(CommonConstants.RootDir))
            {
                try {Directory.Delete(CommonConstants.RootDir,true); }catch {};                
            }

            if (Directory.Exists(CommonConstants.RootDir))
            {
                try
                {
                    var datFiles = Directory.GetFiles(CommonConstants.RootDir).Where(x => x.EndsWith(".dat"));
                    foreach (var datFile in datFiles){File.Delete(datFile);}
                } catch {}
            };
        }

        public void Dispose()
        {
            if (Container != null)
            {
                Container.Dispose();
                Container = null;
            }
        }

    }
}
