﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WcfSignalService
{

    public class SignalService : ISignalService
    {
        private string err = "";
        private int limitDb = 1;

        public bool AddData(DataPack dataPack, int limit = 1)
        {
            if (string.IsNullOrWhiteSpace(dataPack.PackDescription) || limit < 1) return false;
            if (limit > limitDb){limit = limitDb;}

            try
            {
                using (var ctx = new SignalContext())
                {
                    var ds = dataPack.PackDescription;

                    if (!ctx.LineDataSets.Any(x => x.LineDescription == ds))
                    {
                        var nlds = new LineDataSet
                        {
                            LineDescription = ds,
                            DataPacks = new List<DataPack> { dataPack }
                        };
                        ctx.LineDataSets.Add(nlds);
                    }
                    else
                    {
                        var lds = ctx.LineDataSets.FirstOrDefault(x => x.LineDescription == ds);
                        if (lds != null)
                        {
                            lds.DataPacks.Add(dataPack);
                            while (lds.DataPacks.Count > limit)
                            {
                                lds.DataPacks.Remove(lds.DataPacks.First());
                            }
                        }
                    }

                    ctx.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                err += ex.Message;
                return false;
            }

            return true;
        }

        public string[] GetData(string lineId)
        {
            var rezList = new List<string>();
            try
            {
                using (var ctx = new SignalContext())
                {
                    var lds = ctx.LineDataSets;

                    foreach (var ld in lds)
                    {
                        if(ld.DataPacks==null || ld.DataPacks.Count<1)continue;

                        foreach (var dataPack in ld.DataPacks)
                        {
                            var sb = new StringBuilder();
                            sb.AppendLine("PackId: " + dataPack.PackId);
                            sb.AppendLine("LineId: " + dataPack.LineDataSet.LineId);
                            sb.AppendLine("Message: " + dataPack.Message);
                            sb.AppendLine("PackDescription: " + dataPack.PackDescription);
                            sb.AppendLine("IsReturned: " + dataPack.IsReturned);
                            sb.AppendLine("Data: " + dataPack.Data);
                            sb.AppendLine("DateTimeTicks: " + dataPack.DateTimeTicks);
                            sb.AppendLine("LineDescription: " + dataPack.LineDataSet.LineDescription);
                            rezList.Add(sb.ToString());
                        }                    
                    }
                }
            }
            catch (Exception ex)
            {
                string[] errRez = new string[2];
                errRez[0] = "Error Message: " + ex.Message;
                errRez[1] = "InnerException: " + ex.InnerException;
                return errRez;
            }
            return rezList.ToArray();
        }

        public string[] GetLineData(string lineId)
        {
            var rezList = new List<string>();
            try
            {
                using (var ctx = new SignalContext())
                {
                    var lds = ctx.LineDataSets.FirstOrDefault(x => x.LineDescription == lineId);
                    var dataPacks = lds?.DataPacks;

                    foreach (var dataPack in dataPacks)
                    {
                        var sb = new StringBuilder();
                        sb.AppendLine("PackId: " + dataPack.PackId);
                        sb.AppendLine("LineId: " + dataPack.LineDataSet.LineId);
                        sb.AppendLine("Message: " + dataPack.Message);
                        sb.AppendLine("PackDescription: " + dataPack.PackDescription);
                        sb.AppendLine("IsReturned: " + dataPack.IsReturned);
                        sb.AppendLine("Data: " + dataPack.Data);
                        sb.AppendLine("DateTimeTicks: " + dataPack.DateTimeTicks);
                        sb.AppendLine("LineDescription: " + dataPack.LineDataSet.LineDescription);
                        rezList.Add(sb.ToString());
                    }
                }
            }
            catch (Exception ex)
            {
                string[] errRez = new string[2];
                errRez[0] = "Error Message: " + ex.Message;
                errRez[1] = "InnerException: " + ex.InnerException;
                return errRez;
            }
            return rezList.ToArray();
        }

        public string[] GetLines()
        {
            var rezList = new List<string>();
            try
            {
                using (var ctx = new SignalContext())
                {
                    var lds = ctx.LineDataSets;
                    rezList.AddRange(lds.Select(ld => ld.LineDescription));
                }
            }
            catch { return null; }
                
            return rezList.ToArray();
        }

        #region  DB Engine

        public string TestConnection()
        {
            var rez = CreateDb();
            if (rez) { return "DbCreated"; }
            return "DbError:" + err;
        }

        public void SetDbLimit(string passw, int limit)
        {
            if (passw != "admin_pasword" || limitDb < 1) return;
            limitDb = limit;
        }

        public void DropDb(string passw)
        {
            if (passw != "admin_pasword") return;
            var ctx = new SignalContext();
            ctx.Database.Delete();
            ctx.SaveChanges();
        }

        public void CleanDb(string passw)
        {
            if (passw != "admin_pasword") return;
            var ctx = new SignalContext();
            foreach (var lset in ctx.LineDataSets)
            {
                lset.DataPacks.Clear();
            }
            ctx.SaveChanges();
        }

        private bool CreateDb()
        {
            bool rez = false;
            var dpa = new DataPack
            {
                Data = 0.99,
                DateTimeTicks = DateTime.Now.Ticks,
                PackDescription = "Test",
                Message = "TestMessage"
            };
            rez = AddData(dpa, 1);
            return rez;
        }

        #endregion
    }

}
